#!/usr/bin/env/python

#  __all__ = [
#      'alltimeplayerstats',
#      'gamestats',
#      'gamestatsboxscores',
#      'gamestatslineups',
#      'gamestatsrosters',
#      'gamestatsteamstats',
#      'playerprofile',
#      'teamseasonplayerstats',
#      'teamstats'
#  ]



#!/usr/bin/env/python

import pandas as pd


class Endpoint(object):

    def __init__(self, base_url: str):
        self.base_url = base_url
        self.endpoint = None
        self.url = None

    def _fetch_dfs_from_url(self):
        return pd.read_html(self.url)

    def _get_url(self) -> str:
        return f"{self.base_url}{self.endpoint}"


    def _get_endpoint(self) -> str:
        pass



#!/usr/bin/env/python

import sys
import pandas as pd
import requests
from audl.stats.endpoints._base import Endpoint

#  https://audl-stat-server.herokuapp.com/web-api/team-game-stats?limit=20
#  https://audl-stat-server.herokuapp.com/web-api/team-game-stats?limit=20&page=1&year=2021
#  https://audl-stat-server.herokuapp.com/web-api/team-game-stats?limit=20&page=1&year=2021&team=cascades


class TeamGameStatsEndpoint(Endpoint):

    """ 
    Fetching Team Game Stats from https://theaudl.com/stats/team-game-stats
    """

    def __init__(self, season, team):
        """ 
        Parameters
        ----------
        season: string or int
            choices: ['career', 2022, 2019, ..., 2012]
        team: string
            choices: ['all', 'aviators', ..., '<ext_team_id>']


        Examples
        --------
        >>> TeamGameStats('career', 'all')
        >>> TeamGameStats(2022, 'rush')

        """

        super().__init__("https://audl-stat-server.herokuapp.com/web-api/team-game-stats?limit=20")
        self.season = season
        self.team = team

    def get_game_stats(self, show_message=True):
        """ 
        Function that return game statistics

        Parameters
        ----------
        show_message: bool
            True by default. Print page when fetching

        Returns
        -------
        game_stats: pandas.DataFrame

        Examples
        --------
        >>> gamestats = AllTeamGameStats().get_game_stats()
        >>> gamestats = SeasonGameStats(2022).get_game_stats()
        >>> gamestats = TeamSeasonGameStats(2022, 'royal').get_game_stats()
            
        """
        suffix_url = self._get_suffix_url()

        # fetch all games by fetching all pages
        i = 1
        has_games = True
        dfs = pd.DataFrame()
        while(has_games):
            url = f"{self.base_url}&page={i}{suffix_url}"
            #  print(url)
            try:
                page = requests.get(url)
            except:
                print(f'An error occured when fetching the data. Exiting...')
                sys.exit(1)
            results = page.json()
            games = results['stats']
            #  print(games)
            df = pd.json_normalize(games)
            if df.size == 0:
                has_games = False
            i = i + 1 
            dfs = dfs.append(df)

        return dfs


    def _get_suffix_url(self):
        pass
        

class AllTeamGameStats(TeamGameStatsEndpoint):

    def __init__(self):
        super().__init__('career', 'all')

    def _get_suffix_url(self):
        return ""

        
class SeasonGameStats(TeamGameStatsEndpoint):

    def __init__(self, season):
        super().__init__(season, 'all')

    def _get_suffix_url(self):
        return f"&year={self.season}"


class TeamSeasonGameStats(TeamGameStatsEndpoint):

    def __init__(self, season, team):
        super().__init__(season, team)

    def _get_suffix_url(self):
        return f"&year={self.season}&team={self.team}"




#!/usr/bin/env/python

""" Work with Pandas Dataframe """

import pandas as pd
from audl.stats.library.data import teams
from audl.stats.library.data import teams_columns_name
from audl.stats.library.data import team_col_id, team_col_abr, team_col_full_name, team_col_team_name	, team_col_city, team_col_state, team_col_year_founded
from audl.stats.library.data import team_index_full_name, team_index_state, team_index_city, team_index_id, team_index_year_founded, team_index_abbreviation, team_index_team_name


def _get_teams_df():
    return pd.DataFrame(teams, columns=teams_columns_name)


def _find_value_in_column(substring: str, column_name: str) -> list:
    df = _get_teams_df()
    values = []
    for value in list(df[column_name]):
        if substring in value:
            values.append(value)
    return values


def find_teams_containing_substring_in_full_name(substring: str) -> list:
    return _find_value_in_column(substring, team_col_full_name)


def find_teams_containing_substring_in_city(substring: str) -> list:
    return _find_value_in_column(substring, team_col_city)


def _find_row_by_col_args(col_val: str, column_name: str) -> list:
    df = _get_teams_df()
    row = df.loc[df[column_name] == col_val]
    return row.values.flatten().tolist()


def find_team_row_by_full_name(full_name: str) -> list:
    return _find_row_by_col_args(full_name, team_col_full_name)


def find_team_row_by_city(full_name: str) -> list:
    return _find_row_by_col_args(full_name, team_col_city)


def _find_rows_with_same_col_value(shared_value: str, column_name: str):
    df = _get_teams_df()
    rows = []
    for index, value in enumerate(list(df[column_name])):
        if value == shared_value:
            row = df.iloc[index]
            rows.append(list(row))
    return rows


def find_df_teams_with_same_state(state: str):
    data = _find_rows_with_same_col_value(state, team_col_state)
    return pd.DataFrame(data, columns=teams_columns_name)


def find_df_teams_with_same_creation_date(year: str):  # TOFIX: error with 2015
    data = _find_rows_with_same_col_value(year, team_col_year_founded)
    return pd.DataFrame(data, columns=teams_columns_name)


def _find_cell_value_from_col_value(col_value: str, col_name_input: str, col_index_output: int):
    row = _find_row_by_col_args(col_value, col_name_input)
    return row[col_index_output]


def find_team_full_name_from_id(team_id: str) -> str:
    return _find_cell_value_from_col_value(team_id, team_col_id, team_index_full_name)


def find_id_from_team_full_name(full_name: str) -> str:
    return _find_cell_value_from_col_value(full_name, team_col_full_name, team_index_id)


def find_id_from_team_abreviation(abreviation: str) -> str:
    return _find_cell_value_from_col_value(abreviation, team_col_abr, team_index_id)


def find_team_abreviation_from_id(team_id: str) -> str:
    return _find_cell_value_from_col_value(team_id, team_col_id, team_index_abbreviation)


def find_team_full_name_from_team_abreviation(abreviation: str) -> str:
    return _find_cell_value_from_col_value(abreviation, team_col_abr, team_index_full_name)


def find_team_abreviation_from_team_full_name(full_name: str) -> str:
    return _find_cell_value_from_col_value(full_name, team_col_full_name, team_index_abbreviation)


def find_team_name_from_full_name(full_name:str) ->str:
    return _find_cell_value_from_col_value(full_name, team_col_full_name, team_index_team_name)

def get_list_teams_by_name() -> list:
    return _get_dataframe_column_as_list(team_col_full_name)


def get_list_teams_id() -> list:
    return _get_dataframe_column_as_list(team_col_id)




#!/usr/bin/env/python

import pandas as pd
import json 
import requests
import sys

from audl.stats.endpoints._base import Endpoint
from audl.stats.library.parameters import FileName


class PlayerStats(Endpoint):
    """ class that download data from https://theaudl.com/stats/player-stats
        based on [Season], [Per], [Team]
    """

#  https://audl-stat-server.herokuapp.com/web-api/player-stats?limit=20&year=2019&per=possessions&team=aviators
#  https://theaudl.com/stats/player-stats?page=2&per=possessions&team=aviators&year=2019

    def __init__(self, season, per, team):
        """ 
        Parameters
        ----------
        season: string or int
            choices: ['career', 2022, 2019, ..., 2012]
        per: string
            choices: ['total', 'game']
        team: string
            choices: ['all', 'aviators', ..., '<ext_team_id>']


        Examples
        --------
        >>> PlayerStats('career', 'total', 'all')
        >>> PlayerStats(2022, 'minutes', 'breeze')

        """
        self.season = season
        self.per = per
        self.team = team
        super().__init__('https://audl-stat-server.herokuapp.com/web-api/player-stats?limit=20')

    def fetch_table(self, show_message=False):
        """
        Function that fetch stats for all players as dataframe

        Parameters
        ----------
        show_message: bool
            False by default. Print page number when fetching

        Returns
        -------
        players_df: pandas.DataFrame
            dataframe with all players stats for given season, per, team param

        Examples
        --------
        >>> PlayerStats('career', 'total', 'all').fetch_table(show_message=True)

        """
        hasPlayerLeft = True
        all_players = []
        page = 1
        # add all players 
        while(hasPlayerLeft):
            players = self._fetch_page_players_as_json(page)
            if show_message:
                print(page)
            if not players:
                break
            all_players = all_players + players     # concatenating
            page = page + 1

        # turn json as dataframe
        df = pd.DataFrame(all_players)

        return df.drop_duplicates()


    def _get_url(self, page_num: int) -> str:
        """ 
        Function that return the url by concatening the base url with its page number

        Parameters
        ----------
        page_num: int
            page number of the url

        Returns
        -------
        url: string
            complete url of herodoku database requests

        Examples
        --------
        >>> PlayerStats(2019, 'possessions', 'aviators')._get_url(3)
        >>> https://theaudl.com/stats/player-stats?page=3&per=possessions&team=aviators&year=2019


        """
        if self.season == 'career' and self.team == 'all':
            return f"{self.base_url}&page={page_num}&per={self.per}"
        elif self.season == 'career':
            return f"{self.base_url}&page={page_num}&per={self.per}&team={self.team}"
        elif self.team == 'all':
            return f"{self.base_url}&page={page_num}&year={self.season}&per={self.per}"
        return f"{self.base_url}&page={page_num}&year={self.season}&per={self.per}&team={self.team}"

    def _fetch_page_players_as_json(self, page_num):
        """
        Function that fetch players in page as json 

        Parameters
        ----------
        page_num: int
            page number of the url

        Returns
        -------
        player_json: json
            json document of player page request

        Examples
        --------
        >>> PlayerStats(2019, 'possessions', 'aviators')._fetch_page_players_as_json(3)

        """
        try:
            url = self._get_url(page_num)
            page = requests.get(url)
            results = page.json()
            players = results['stats']
        except: 
            print(f'An error has occured when fetching the data from {url}')
            sys.exit(1)
            
        return players


    # TODO: create Download class
    def download_stats_as_dataframe(self, file_path_name, show_message=True):
        """ 
        Function that download players stats as csv file

        Parameters
        ----------
        file_path_name: string 
            path were the file should be downloaded
        show_message: bool
            True by default. Print message when page has been fetched

        Returns
        -------
        flag: bool
            True is file has been downloaded successfully, else False

        Examples
        --------
        >>> PlayerStats(2019, 'possessions', 'aviators').download_stats_as_dataframe('../database/filename.csv')

        """
        flag = False
        try:
            df = self.fetch_table()
            df.to_csv(file_path_name, sep=',', index=False)
            print(f'Downloaded csv file at {file_path_name}')
            flag = True
        except: 
            print('An error has occured when saving the file as csv')
        return flag 





#!/usr/bin/env/python

class Division:
    altantic = 'Atlantic'
    central = 'Central'
    west = 'West'
    canada = 'Canada Cup'


class StatisticsDescription:
    YR = 'Year'
    G = 'Games Played'
    PP = 'Points Played'
    TM = 'Team'
    AST = 'Assists'
    GLS = 'Goals'
    BLK = 'Blocks'
    PLUS_MINUS = 'Plus Minus'
    CMP = 'Completions'
    CMP_PERC = 'Completion Percentage'
    TY = 'Throwing Yards'
    RY = 'Receiving Yards'
    HA = 'Hockey Assists'
    T = 'Throwaways'
    S = 'Stall'
    C = 'Callahan'
    D = 'Drops'
    OPP = 'Offensive Points Played'
    DPP = 'Defensive Points Played'
    P = 'Pulls'


class StatisticAbbreviation:
    stat_col_year = 'YR'
    stat_col_games_played = 'G'
    stat_col_points_played = 'PP'
    stat_col_team = 'TM'
    stat_col_assists = 'AST'
    stat_col_goals = 'GLS'
    stat_col_blocks = 'BLK'
    stat_col_plus_minus = '+/-'
    stat_col_completions = 'CMP'
    stat_col_completion_perc = 'CMP%'
    stat_col_throwing_yards = 'TY'
    stat_col_receiving_yards = 'RY'
    stat_col_hockey_assists = 'HA'
    stat_col_throwaways = 'T'
    stat_col_stalls = 'S'
    stat_col_callahans = 'C'
    stat_col_drops = 'D'
    stat_col_offensive_points_played = 'OPP'
    stat_col_defensive_points_played = 'DPP'
    stat_col_pulls = 'P'


class FileName:
    alltimeplayer = 'AllTimePlayerStats'
    seasonplayerstats = 'SeasonPlayerStats'
    seasonschedule = 'SeasonSchedule'

#######################################################################
#                         TeamSeasonSchedule                          #
#######################################################################

team_season_schedule_index_away	 = 0
team_season_schedule_index_home	 = 1
team_season_schedule_index_time = 2
team_season_schedule_index_stadium = 3
team_season_schedule_index_game_id = 4
team_season_schedule_index_home_score = 5
team_season_schedule_index_away_score = 6

game_schedule_columns_name = [
    "Away",
    "Home",
    "Time",
    "Stadium",
    "Game ID",
    "Away Score",
    "Home Score"
]

#######################################################################
#                           Season Schedule                           #
#######################################################################

NUM_WEEKS_IN_SEASON = 12


#######################################################################
#                             Box Scores                              #
#######################################################################

box_scores_columns_names = ["Team", "Q1", "Q2", "Q3", "Q4", "T"]

# Absolute time in seconds for each quarter
# Regular time: 12 min per quarter
# 1rst OT: 5 minutes
# 2nd OT: sudden death
quarters_clock_dict = {
    "Q1_start": 1,      # 0 min
    "Q1_end": 720,      # 12 min
    "Q2_start": 721,    # 12 min
    "Q2_end": 1440,     # 24 min
    "Q3_start": 1441,   # 24 min
    "Q3_end": 2160,     # 36 min
    "Q4_start": 2161,   # 36 min
    "Q4_end": 2880,        # 48 min
    "OT1_start": 2881,     # 48 min
    "OT1_end": 3180,       # 53 min
    "OT2_start": 3181,     # 53 min
    #  "OT2_end": 0,       # ? min
}

#######################################################################
#                          Games Parameters                           #
#######################################################################

# Types of throws and their id :\t on heroku server


class GameEventAction:
    # Types of events defined on heroku server
    # Block
    # Dish
    # Dump
    # Huck throwaway
    # Pass
    # Pull
    # Score
    # Swing
    pass


class TeamStatsName:
    Completions = 'Completions'
    Hucks = 'Hucks'
    Offensive_holds = 'Offensive Holds'
    Defensive_breaks = 'Defensive Breaks'
    Red_zone_possessions = 'Red Zone Posessions'
    Hucks = 'Hucks'
    Turnovers = 'Turnovers'
    Blocks = 'Blocks'
    Turnovers = 'Turnovers'


team_stats_row_names = [
    TeamStatsName.Completions,
    TeamStatsName.Hucks,
    TeamStatsName.Offensive_holds,
    TeamStatsName.Defensive_breaks,
    TeamStatsName.Red_zone_possessions,
    TeamStatsName.Blocks,
    TeamStatsName.Turnovers,
]

team_stats_perc_columns_names = [
    "",
    "Successful",
    "Opportunities",
    "Percentage"
]

team_roster_columns_name = [
    "identification_num",
    "first_name",
    "last_name",
    "jersey_number",
    "player_id"

]

# \t: number mapings
# \r: receiver
# \l: lineup
# \x: disc absolute position in x
# \y: disc absolut position in y
# \s: ???




team_points_by_points_columns_names = [
    "Current Score",
    "Line",
    "Clock",
    "Point Duration",
    "Quarter",
    "Lineup"
]

play_by_play_columns_names = [
    "Score",
    "Offensive Team",
    "Defensive Team",
    "O-Line",
    "D-Line",
    "Quarter",
    "Point Duration",
    "Clock",
    "Outcome"
]

class HerokuPlay:
    OLineIndex = 1
    DLineIndex = 2
    Pull = 3
    Block = 5
    ThrowawayCaused = 8
    Throwaway = 9
    EndOfQ1 = 23
    EndOfQ2 = 24
    EndOfQ3 = 25
    EndOfQ4 = 26
    PassCompleted = 20
    ScoredOn = 21
    Goal = 22
    TimeoutDefense = 40
    TimeoutOffense = 41

game_event_dict = {
    1: 'Offensive Point', # l is the lineup: list of player_id
    2: 'Defensive Point', # l is the lineup: list of player_id
    3: 'Pull', # r is the person who performed the pull
    5: 'Block', # r is the person who made the block
    8: 'Throw away',  # check previous player
    9: 'Throwaway Caused', # check 13
    #  10: '', 
    #  11: '', # always between 8,9
    12: 'call on the field?', # c is call type?
    13: 'switch from off to def?', # check 9
    14: 'Time Out called (we are on off)', # s is 
    15: 'Time Out called (we are on def)', # s is ? (not the time)
    19: 'Pass dropped', # r is the person who dropped the catch (thrower is prev)
    20: 'Pass Completed', # r is the person who caught the catch (thrower is prev)
    21: 'They score', # s is 
    22: 'We Score', # r is the person who scored the point
    23: 'End of Q1', 
    24: 'End of Q2', 
    25: 'End of Q3', 
    26: 'End of Q4', 
    #  27: 'End of OT1', 
    #  28: 'End of OT2', 
    40: 'Timeout: defense unit comes to the field', # l is lineup
    41: 'Timeout: offense unit comes to the field', # l is lineup
    #  42: '', # s is 
    #  43: '', # s is 
    44: 'Offside', # r is the person who made the offside (if we don't know who it is, then we dont have row['r'])
    50: 'Start Game', 
}


# sequence: 13, 54, 53, 21 (we)
        
        






#######################################################################
#                            Miscellaneous                            #
#######################################################################
season_dict = {
    2021: '1',
    2019: '2',
    2018: '3',
    2017: '4',
    2015: '5',
    2016: '6',
    2014: '7',
    2013: '8',
    2012: '9',
}



#!/usr/bin/env/python

import pandas as pd
import requests
import json 
import sys

from bs4 import BeautifulSoup

from audl.stats.endpoints._base import Endpoint
from audl.stats.library.parameters import game_schedule_columns_name
from audl.stats.library.parameters import FileName

#  https://audl-stat-server.herokuapp.com/web-api/games?limit=10&years=2022&teamID=alleycats



class ScheduleEndpoint(Endpoint):

    def __init__(self):
        ""
        super().__init__("https://audl-stat-server.herokuapp.com/web-api/games?limit=10")

    def _get_prefix_url(self):
        pass


    def get_schedule(self):
        """ 
        Function that fetch all games from prefix url.  This function will be called from the children class

        Returns
        -------
        schedule: pandas.DataFrame
            dataframe with all schedule infos


        """
        # get prefix url
        prefix_url = self._get_prefix_url()

        # fetch all games by fetching all pages
        i = 1
        has_games = True
        dfs = pd.DataFrame()
        while(has_games):
            url = f"{prefix_url}&page={i}"
            print(url)
            try:
                page = requests.get(url)
            except:
                print(f'An error occured when fetching the data. Exiting...')
                sys.exit(1)
            results = page.json()
            games = results['games']
            df = pd.json_normalize(games)
            if df.size == 0:
                has_games = False
            i = i + 1 
            dfs = dfs.append(df)

        return dfs



class AllSchedule(ScheduleEndpoint):

    def __init__(self):
        """ 
        Examples
        --------
        >>> schedule = AllSchedule().get_schedule()

        """
        super().__init__()

    def _get_prefix_url(self):
        return self.base_url


class SeasonSchedule(ScheduleEndpoint):

    def __init__(self, season):
        """ 
        Examples
        --------
        >>> schedule = SeasonSchedule(2022).get_schedule()

        """
        super().__init__()
        self.season = season

    def _get_prefix_url(self):
        return f"{self.base_url}&years={self.season}"


class TeamSeasonSchedule(ScheduleEndpoint):

    def __init__(self, season, team):
        """ 
        Examples
        --------
        >>> schedule = TeamSeasonSchedule(2022, 'breeze').get_schedule()

        """
        super().__init__()
        self.season = season
        self.team = team

    def _get_prefix_url(self):
        return f"{self.base_url}&years={self.season}&teamID={self.team}"

class TeamSeasonAgainstOpponentSchedule(ScheduleEndpoint):

    def __init__(self, season, team, opponent):
        """ 
        Examples
        --------
        >>> schedule = TeamSeasonAgainstOpponentSchedule(2022, 'royal', 'rush').get_schedule()

        """
        super().__init__()
        self.season = season
        self.team = team
        self.opponent = opponent

    def _get_prefix_url(self):
        return f"{self.base_url}&years={self.season}&teamID={self.team}&opposingTeamID={self.opponent}"

